'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.fetchUserInfoWithUserId = fetchUserInfoWithUserId;
exports.fetchUserInfoFromServerWithUserId = fetchUserInfoFromServerWithUserId;
exports.fetchUserInfoFromRedisWithUserId = fetchUserInfoFromRedisWithUserId;
exports.setUserInfoForRedis = setUserInfoForRedis;

var _request = require('request');

var _request2 = _interopRequireDefault(_request);

var _co = require('co');

var _co2 = _interopRequireDefault(_co);

var _fetchDataFromHomeRpc = require('../rpc/fetchDataFromHomeRpc');

var _fetchDataFromHomeRpc2 = _interopRequireDefault(_fetchDataFromHomeRpc);

var _client = require('../redis/client');

var _client2 = _interopRequireDefault(_client);

var _kRedisKeys = require('../redis/kRedisKeys');

var _config = require('../config');

var _config2 = _interopRequireDefault(_config);

var _fetchDataFromRpc = require('../rpc/fetchDataFromRpc');

var _fetchDataFromRpc2 = _interopRequireDefault(_fetchDataFromRpc);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var debug = require('debug')('ibe-utils:' + __filename.split('/').slice(-3).join('/'));

function* fetchUserInfoWithUserId(userId) {
  console.log('fetchUserInfoWithUserId:L11:', userId);
  if (!userId) {
    return Promise.reject(new Error('userId不能为空'));
  }
  var userInfo = void 0;
  try {
    userInfo = yield fetchUserInfoFromRedisWithUserId(userId);
    console.log('userInfo:L18:', userInfo.objectId);
  } catch (e) {
    try {
      userInfo = yield fetchUserInfoFromServerWithUserId(userId);
      console.log('userInfo:L22:', userInfo.objectId);
    } catch (error) {
      return Promise.reject(error);
    }
  }
  return Promise.resolve(userInfo);
}
function* fetchUserInfoFromServerWithUserId(userId) {
  try {
    var _ref = yield (0, _fetchDataFromRpc2.default)(_config2.default.url.ibeHome + 'rpc/v1.0/user/' + userId + '.json');

    var userInfo = _ref.userInfo;

    console.log('userInfo:L33', userInfo.objectId);
    yield setUserInfoForRedis(userInfo);
    return Promise.resolve(userInfo);
  } catch (e) {
    return Promise.reject(e);
  }
}
function* fetchUserInfoFromRedisWithUserId(userId) {
  var key = (0, _kRedisKeys.kFnRedisKeyUserInfoWithUserId)(userId);
  var userInfo = yield _client2.default.get(key);
  if (userInfo) {
    return Promise.resolve(JSON.parse(userInfo));
  } else {
    return Promise.reject(new Error());
  }
}

function* setUserInfoForRedis(userInfo) {
  var key = (0, _kRedisKeys.kFnRedisKeyUserInfoWithUserId)(userInfo.objectId);
  yield _client2.default.set(key, JSON.stringify(userInfo));
  // 一天过期
  return yield _client2.default.expire(key, 60 * 60 * 24);
}