'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.fetchWeixinMpInfo = fetchWeixinMpInfo;

var _kRedisKeys = require('../redis/kRedisKeys');

var _config = require('../config');

var _config2 = _interopRequireDefault(_config);

var _client = require('../redis/client');

var _client2 = _interopRequireDefault(_client);

var _fetchDataFromRpc = require('../rpc/fetchDataFromRpc');

var _fetchDataFromRpc2 = _interopRequireDefault(_fetchDataFromRpc);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var debug = require('debug')('ibe-utils:' + __filename.split('/').slice(-3).join('/'));


var isUpdating = false;
/**
 * 从redis中获取weixin配置,
 * 当检测到 token 快过期的时候,会自动更新。
 * @returns {Promise}
 */
function* fetchWeixinMpInfo() {
  var configStr = yield _client2.default.get(_kRedisKeys.kRedisKeySystemConfig);
  var systemConfig = JSON.parse(configStr);

  var weixinmp = systemConfig.weixinmp;
  var expires = weixinmp.expires;

  /**
   * 如果过期时间快到了
   */
  var t = +new Date();
  // 当过期时间还有2分钟时,更新配置
  if (expires - t <= 2 * 60 * 1000) {
    debug('开始:更新微信token');
    weixinmp = yield updateWeixinMpInfo();
    debug('完成:更新微信token');
  } else {
    debug('微信token没有过期,不需要更新');
  }

  return Promise.resolve(weixinmp);
}

function* updateWeixinMpInfo() {
  if (isUpdating) {
    return Promise.resolve(true);
  }
  isUpdating = true;

  var url = _config2.default.url.ibeHome + 'rpc/v1.0/system/weixinmp/update.json';
  var payload = yield (0, _fetchDataFromRpc2.default)(url);

  isUpdating = false;

  return Promise.resolve(payload.weixinmp);
}