'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (client) {
  var result = {};
  methods.forEach(function (name) {
    result[name] = function* () {
      for (var _len = arguments.length, arg = Array(_len), _key = 0; _key < _len; _key++) {
        arg[_key] = arguments[_key];
      }

      return new Promise(function (resolve, reject) {
        client[name].apply(client, arg.concat([function (e, result) {
          if (e) {
            return reject(e);
          }
          resolve(result);
        }]));
      });
    };
  });
  return result;
};

var methods = [
// region ---- 通用 ----
'del',
/**
 * 过期时间设置,只对顶级 key 生效,
 * 所以比如想要设置 hash 某个 item 的过期时间是不可行的. [refer](https://github.com/antirez/redis/issues/167)
 */
'expire', 'exists',
// endregion

//region ---- string ----
'get', 'set', 'incr',
// endregion

// region ---- hash ----
'hset', 'hget', 'hmget', 'hmset', 'hgetall', 'hincrby', 'hexists',
// endregion

// region ---- SortedSet 有序集合 ----
//

/** ZADD key score member [[score member] [score member] ...]
 * 将一个或多个 member 元素及其 score 值加入到有序集 key 当中。
 * 如果某个 member 已经是有序集的成员，那么更新这个 member 的 score 值，并通过重新插入这个 member 元素，来保证该 member 在正确的位置上。
 * score 值可以是整数值或双精度浮点数。
 * 如果 key 不存在，则创建一个空的有序集并执行 ZADD 操作。
 * 当 key 存在但不是有序集类型时，返回一个错误。
 *
 * 返回值:
 * 被成功添加的新成员的数量，不包括那些被更新的、已经存在的成员。
 * zadd(key,score,value);
 *
 */
'zadd',
/**
 * ZCARD key
 * 返回有序集 key 的基数。
 *
 * 返回值:
 * 当 key 存在且是有序集类型时，返回有序集的基数(集合中元素的数量)。
 * 当 key 不存在时，返回 0 。
 */
'zcard',
/**
 * ZCOUNT key min max
 * 返回有序集 key 中， score 值在 min 和 max 之间(默认包括 score 值等于 min 或 max )的成员的数量。
 * 关于参数 min 和 max 的详细使用方法，请参考 ZRANGEBYSCORE 命令。
 */
'zcount',
/**
 * ZINCRBY key increment member
 * 为有序集 key 的成员 member 的 score 值加上增量 increment 。
 * 可以通过传递一个负数值 increment ，让 score 减去相应的值，比如 ZINCRBY key -5 member ，就是让 member 的 score 值减去 5 。
 * 当 key 不存在，或 member 不是 key 的成员时， ZINCRBY key increment member 等同于 ZADD key increment member 。
 *  当 key 不是有序集类型时，返回一个错误。
 *  score 值可以是整数值或双精度浮点数。
 */
'zincrby',
// --- 区间操作 ---
/**
 * ZRANGE key start stop [WITHSCORES]
 * 返回有序集 key 中，指定区间内的成员。
 * 其中成员的位置按 score 值递增(从小到大)来排序。
 * 具有相同 score 值的成员按字典序(lexicographical order )来排列。
 * 如果你需要成员按 score 值递减(从大到小)来排列，请使用 ZREVRANGE 命令。
 * 下标参数 start 和 stop 都以 0 为底，也就是说，以 0 表示有序集第一个成员，以 1 表示有序集第二个成员，以此类推。
 * 你也可以使用负数下标，以 -1 表示最后一个成员， -2 表示倒数第二个成员，以此类推。
 * 超出范围的下标并不会引起错误。
 * 比如说，当 start 的值比有序集的最大下标还要大，或是 start > stop 时， ZRANGE 命令只是简单地返回一个空列表。
 * 另一方面，假如 stop 参数的值比有序集的最大下标还要大，那么 Redis 将 stop 当作最大下标来处理。
 * 可以通过使用 WITHSCORES 选项，来让成员和它的 score 值一并返回，返回列表以 value1,score1, ..., valueN,scoreN 的格式表示。
 * 客户端库可能会返回一些更复杂的数据类型，比如数组、元组等。
 */
'zrange',
/**
 * 同上,
 * */
'zrevrange',
/**
 * ZRANGEBYSCORE key min max [WITHSCORES] [LIMIT offset count]
 * 返回有序集 key 中，所有 score 值介于 min 和 max 之间(包括等于 min 或 max )的成员。有序集成员按 score 值递增(从小到大)次序排列。
 * 具有相同 score 值的成员按字典序(lexicographical order)来排列(该属性是有序集提供的，不需要额外的计算)。
 * 可选的 LIMIT 参数指定返回结果的数量及区间(就像SQL中的 SELECT LIMIT offset, count )，注意当 offset 很大时，定位 offset 的操作可能需要遍历整个有序集，此过程最坏复杂度为 O(N) 时间。
 * 可选的 WITHSCORES 参数决定结果集是单单返回有序集的成员，还是将有序集成员及其 score 值一起返回。
 * 该选项自 Redis 2.0 版本起可用。
 * 区间及无限
 * min 和 max 可以是 -inf 和 +inf ，这样一来，你就可以在不知道有序集的最低和最高 score 值的情况下，使用 ZRANGEBYSCORE 这类命令。
 * 默认情况下，区间的取值使用闭区间 (小于等于或大于等于)，你也可以通过给参数前增加 ( 符号来使用可选的开区间 (小于或大于)。
 */
'zrangebyscore', 'zrevrangebyscore',
/**
 * ZRANK key member
 * 返回有序集 key 中成员 member 的排名。其中有序集成员按 score 值递增(从小到大)顺序排列。
 * 排名以 0 为底，也就是说， score 值最小的成员排名为 0 。
 * 使用 ZREVRANK 命令可以获得成员按 score 值递减(从大到小)排列的排名。
 */
'zrank', 'zrevrank',
/**
 * ZREM key member [member ...]
 * 移除有序集 key 中的一个或多个成员，不存在的成员将被忽略。
 * 当 key 存在但不是有序集类型时，返回一个错误。
 */
'zrem', 'zremrangebyrank', 'zremrangebyscore',
/**
 * 返回有序集 key 中，成员 member 的 score 值。
 * 如果 member 元素不是有序集 key 的成员，或 key 不存在，返回 nil 。
 */
'zscore',
/**
 * yield redisClient.zinterstore(key1, 2, key1, key2, 'weights', 1, 1, 'aggregate', 'max');
 * 并集 aggergate
 * ZUNIONSTORE destination numkeys key [key ...] [WEIGHTS weight [weight ...]] [AGGREGATE SUM|MIN|MAX]
 * 计算给定的一个或多个有序集的并集，其中给定 key 的数量必须以 numkeys 参数指定，并将该并集(结果集)储存到 destination 。
 * 默认情况下，结果集中某个成员的 score 值是所有给定集下该成员 score 值之 和 。
 * WEIGHTS
 * 使用 WEIGHTS 选项，你可以为 每个 给定有序集 分别 指定一个乘法因子(multiplication factor)，每个给定有序集的所有成员的 score 值在传递给聚合函数(aggregation function)之前都要先乘以该有序集的因子。
 * 如果没有指定 WEIGHTS 选项，乘法因子默认设置为 1 。
 * AGGREGATE
 * 使用 AGGREGATE 选项，你可以指定并集的结果集的聚合方式。
 * 默认使用的参数 SUM ，可以将所有集合中某个成员的 score 值之 和 作为结果集中该成员的 score 值；使用参数 MIN ，可以将所有集合中某个成员的 最小 score 值作为结果集中该成员的 score 值；而参数 MAX 则是将所有集合中某个成员的 最大 score 值作为结果集中该成员的 score 值。
 */
'zunionstore',
/**
 * 交集
 * ZINTERSTORE destination numkeys key [key ...] [WEIGHTS weight [weight ...]] [AGGREGATE SUM|MIN|MAX]
 * 计算给定的一个或多个有序集的交集，其中给定 key 的数量必须以 numkeys 参数指定，并将该交集(结果集)储存到 destination 。
 * 默认情况下，结果集中某个成员的 score 值是所有给定集下该成员 score 值之和.
 * 关于 WEIGHTS 和 AGGREGATE 选项的描述，参见 ZUNIONSTORE 命令。
 */
'zinterstore',
/**
 * TODO: 以下关于 有序列表 的内容需要学习一下.
 */
'zscan', 'zrangebylex', 'zlexcount', 'zremrangebylex',
// endregion

// region Set 集合
'sadd', 'scard',
/**
 * SDIFF key [key ...]
 * 返回一个集合的全部成员，该集合是所有给定集合之间的差集。
 * 不存在的 key 被视为空集。
 * 返回值:
 * 一个包含差集成员的列表。
 */
'sdiff',
/**
 * SDIFFSTORE destination key [key ...]
 * 这个命令的作用和 SDIFF 类似，但它将结果保存到 destination 集合，而不是简单地返回结果集。
 * 如果 destination 集合已经存在，则将其覆盖。
 * destination 可以是 key 本身。
 * 返回值:
 * 结果集中的元素数量。
 */
'sdiffstore',
/**
 * SINTER key [key ...]
 * 返回一个集合的全部成员，该集合是所有给定集合的交集。
 * 不存在的 key 被视为空集。
 * 当给定集合当中有一个空集时，结果也为空集(根据集合运算定律)。
 * 返回值:
 * 交集成员的列表。
 */
'sinter', 'sinterstore',
/**
 * SUNION key [key ...]
 *
 * 返回一个集合的全部成员，该集合是所有给定集合的并集。
 *
 * 不存在的 key 被视为空集。
 *
 * 可用版本：
 * >= 1.0.0
 * 时间复杂度:
 * O(N)， N 是所有给定集合的成员数量之和。
 * 返回值:
 * 并集成员的列表。
  */
'sunion', 'SUNIONSTORE',
/**
 * SISMEMBER key member
 * 判断 member 元素是否集合 key 的成员。
 * 返回值:
 * 如果 member 元素是集合的成员，返回 1 。
 * 如果 member 元素不是集合的成员，或 key 不存在，返回 0 。
 */
'sismember',
/**
 * SMEMBERS key
 * 返回集合 key 中的所有成员。
 * 不存在的 key 被视为空集合。
 */
'smembers',
/**
 * SMOVE source destination member
 * 将 member 元素从 source 集合移动到 destination 集合。
 * SMOVE 是原子性操作。
 * 如果 source 集合不存在或不包含指定的 member 元素，则 SMOVE 命令不执行任何操作，仅返回 0 。否则， member 元素从 source 集合中被移除，并添加到 destination 集合中去。
 * 当 destination 集合已经包含 member 元素时， SMOVE 命令只是简单地将 source 集合中的 member 元素删除。
 * 当 source 或 destination 不是集合类型时，返回一个错误。
 * 可用版本：
 * >= 1.0.0
 * 时间复杂度:
 * O(1)
 * 返回值:
 * 如果 member 元素被成功移除，返回 1 。
 * 如果 member 元素不是 source 集合的成员，并且没有任何操作对 destination 集合执行，那么返回 0 。
 */
'smove',
/**
 * SPOP key
 * 移除并返回集合中的一个随机元素。
 * 如果只想获取一个随机元素，但不想该元素从集合中被移除的话，可以使用 SRANDMEMBER 命令。
 * 可用版本：
 * >= 1.0.0
 * 时间复杂度:
 * O(1)
 * 返回值:
 * 被移除的随机元素。
 * 当 key 不存在或 key 是空集时，返回 nil 。
 */
'spop',
/**
 * SRANDMEMBER key [count]
 * 如果命令执行时，只提供了 key 参数，那么返回集合中的一个随机元素。
 * 从 Redis 2.6 版本开始， SRANDMEMBER 命令接受可选的 count 参数：
 * 如果 count 为正数，且小于集合基数，那么命令返回一个包含 count 个元素的数组，数组中的元素各不相同。如果 count 大于等于集合基数，那么返回整个集合。
 * 如果 count 为负数，那么命令返回一个数组，数组中的元素可能会重复出现多次，而数组的长度为 count 的绝对值。
 * 该操作和 SPOP 相似，但 SPOP 将随机元素从集合中移除并返回，而 SRANDMEMBER 则仅仅返回随机元素，而不对集合进行任何改动。
 *
 * 可用版本：
 * >= 1.0.0
 * 时间复杂度:
 * 只提供 key 参数时为 O(1) 。
 * 如果提供了 count 参数，那么为 O(N) ，N 为返回数组的元素个数。
 * 返回值:
 * 只提供 key 参数时，返回一个元素；如果集合为空，返回 nil 。
 * 如果提供了 count 参数，那么返回一个数组；如果集合为空，返回空数组。
 */
'srandmember',
/**
 * 移除集合 key 中的一个或多个 member 元素，不存在的 member 元素会被忽略。
 *
 * 当 key 不是集合类型，返回一个错误。
 *
 * 在 Redis 2.4 版本以前， SREM 只接受单个 member 值。
 * 可用版本：
 * >= 1.0.0
 * 时间复杂度:
 * O(N)， N 为给定 member 元素的数量。
 * 返回值:
 * 被成功移除的元素的数量，不包括被忽略的元素。
 */
'srem', 'sscan'];
// endregion


;