'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.generateUIDNotMoreThan1million = generateUIDNotMoreThan1million;
exports.createSignature = createSignature;
exports.checkSignature = checkSignature;

var _crypto = require('crypto');

var _crypto2 = _interopRequireDefault(_crypto);

var _client = require('./redis/client');

var _client2 = _interopRequireDefault(_client);

var _kRedisKeys = require('./redis/kRedisKeys');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * 获取随机字符串
 * @returns {string}
 */
function generateUIDNotMoreThan1million() {
  return ("0000" + (Math.random() * Math.pow(36, 4) << 0).toString(36)).slice(-4);
}
/**
 * 创建签名
 * @returns {{signature: string, nonce: string, timestamp: number}}
 */
/**
 * 系统间请求签名.
 */
function* createSignature() {
  var token = yield _client2.default.get(_kRedisKeys.kRedisKeySignatureToken);

  var sha1 = _crypto2.default.createHash('sha1');
  var nonce = generateUIDNotMoreThan1million();
  var timestamp = +new Date();

  var array = [token, nonce, timestamp].sort(function (a, b) {
    return a - b;
  });
  sha1.update(array.join(','));
  var signature = sha1.digest('hex');
  return {
    signature: signature,
    nonce: nonce,
    timestamp: timestamp
  };
}
/**
 * 校验签名
 * @param signature 签名
 * @param nonce 随机字符串
 * @param timestamp 时间戳
 * @returns {boolean}
 */
function* checkSignature(_ref) {
  var signature = _ref.signature;
  var nonce = _ref.nonce;
  var timestamp = _ref.timestamp;

  var token = yield _client2.default.get(_kRedisKeys.kRedisKeySignatureToken);
  var sha1 = _crypto2.default.createHash('sha1');
  var array = [token, nonce, timestamp].sort(function (a, b) {
    return a - b;
  });
  sha1.update(array.join(','));
  return signature === sha1.digest('hex');
}